@@ -41,10 +41,18 @@
 #include <chrono>
 #include <memory>
 
-#include <openssl/ssl.h>
+#include "ssl_compat.h"
+
+#ifdef NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/ssl.h>
+#else // !NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <openssl/ssl.h>
+#endif // !NGHTTP2_OPENSSL_IS_WOLFSSL
 
 #include <ev.h>
 
+#define NGHTTP2_NO_SSIZE_T
 #include <nghttp2/nghttp2.h>
 
 #include "llhttp.h"
@@ -77,7 +85,7 @@ struct Config {
   int64_t encoder_header_table_size;
   size_t padding;
   size_t max_concurrent_streams;
-  ssize_t peer_max_concurrent_streams;
+  size_t peer_max_concurrent_streams;
   int multiply;
   // milliseconds
   ev_tstamp timeout;
@@ -137,7 +145,7 @@ struct ContinueTimer {
 struct Request {
   // For pushed request, |uri| is empty and |u| is zero-cleared.
   Request(const std::string &uri, const http_parser_url &u,
-          const nghttp2_data_provider *data_prd, int64_t data_length,
+          const nghttp2_data_provider2 *data_prd, int64_t data_length,
           const nghttp2_priority_spec &pri_spec, int level = 0);
   ~Request();
 
@@ -180,7 +188,7 @@ struct Request {
   int64_t response_len;
   nghttp2_gzip *inflater;
   std::unique_ptr<HtmlParser> html_parser;
-  const nghttp2_data_provider *data_prd;
+  const nghttp2_data_provider2 *data_prd;
   size_t header_buffer_size;
   int32_t stream_id;
   int status;
@@ -246,7 +254,7 @@ struct HttpClient {
   bool all_requests_processed() const;
   void update_hostport();
   bool add_request(const std::string &uri,
-                   const nghttp2_data_provider *data_prd, int64_t data_length,
+                   const nghttp2_data_provider2 *data_prd, int64_t data_length,
                    const nghttp2_priority_spec &pri_spec, int level = 0);
 
   void record_start_time();
