@@ -33,7 +33,6 @@
 #include <vector>
 #include <chrono>
 
-#include "shrpx_config.h"
 #include "shrpx_log_config.h"
 #include "tls.h"
 #include "template.h"
@@ -45,48 +44,59 @@ using namespace nghttp2;
 
 #define LOG_ENABLED(SEVERITY) (ENABLE_LOG && shrpx::Log::log_enabled(SEVERITY))
 
-#define LOG(SEVERITY) shrpx::Log(SEVERITY, __FILE__, __LINE__)
+#ifdef __FILE_NAME__
+#  define NGHTTP2_FILE_NAME __FILE_NAME__
+#else // !__FILE_NAME__
+#  define NGHTTP2_FILE_NAME __FILE__
+#endif // !__FILE_NAME__
+
+#define LOG(SEVERITY) shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)
 
 // Listener log
 #define LLOG(SEVERITY, LISTEN)                                                 \
-  (shrpx::Log(SEVERITY, __FILE__, __LINE__) << "[LISTEN:" << LISTEN << "] ")
+  (shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)                           \
+   << "[LISTEN:" << LISTEN << "] ")
 
 // Worker log
 #define WLOG(SEVERITY, WORKER)                                                 \
-  (shrpx::Log(SEVERITY, __FILE__, __LINE__) << "[WORKER:" << WORKER << "] ")
+  (shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)                           \
+   << "[WORKER:" << WORKER << "] ")
 
 // ClientHandler log
 #define CLOG(SEVERITY, CLIENT_HANDLER)                                         \
-  (shrpx::Log(SEVERITY, __FILE__, __LINE__)                                    \
+  (shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)                           \
    << "[CLIENT_HANDLER:" << CLIENT_HANDLER << "] ")
 
 // Upstream log
 #define ULOG(SEVERITY, UPSTREAM)                                               \
-  (shrpx::Log(SEVERITY, __FILE__, __LINE__) << "[UPSTREAM:" << UPSTREAM        \
-                                            << "]"                             \
-                                               " ")
+  (shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)                           \
+   << "[UPSTREAM:" << UPSTREAM << "] ")
 
 // Downstream log
 #define DLOG(SEVERITY, DOWNSTREAM)                                             \
-  (shrpx::Log(SEVERITY, __FILE__, __LINE__)                                    \
+  (shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)                           \
    << "[DOWNSTREAM:" << DOWNSTREAM << "] ")
 
 // Downstream connection log
 #define DCLOG(SEVERITY, DCONN)                                                 \
-  (shrpx::Log(SEVERITY, __FILE__, __LINE__) << "[DCONN:" << DCONN << "] ")
+  (shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)                           \
+   << "[DCONN:" << DCONN << "] ")
 
 // Downstream HTTP2 session log
 #define SSLOG(SEVERITY, HTTP2)                                                 \
-  (shrpx::Log(SEVERITY, __FILE__, __LINE__) << "[DHTTP2:" << HTTP2 << "] ")
+  (shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)                           \
+   << "[DHTTP2:" << HTTP2 << "] ")
 
 // Memcached connection log
 #define MCLOG(SEVERITY, MCONN)                                                 \
-  (shrpx::Log(SEVERITY, __FILE__, __LINE__) << "[MCONN:" << MCONN << "] ")
+  (shrpx::Log(SEVERITY, NGHTTP2_FILE_NAME, __LINE__)                           \
+   << "[MCONN:" << MCONN << "] ")
 
 namespace shrpx {
 
 class Downstream;
 struct DownstreamAddr;
+struct LoggingConfig;
 
 enum SeverityLevel { INFO, NOTICE, WARN, ERROR, FATAL };
 
@@ -100,12 +110,20 @@ public:
   Log &operator<<(const char *s);
   Log &operator<<(const StringRef &s);
   Log &operator<<(const ImmutableString &s);
-  Log &operator<<(int16_t n) { return *this << static_cast<int64_t>(n); }
-  Log &operator<<(int32_t n) { return *this << static_cast<int64_t>(n); }
-  Log &operator<<(int64_t n);
-  Log &operator<<(uint16_t n) { return *this << static_cast<uint64_t>(n); }
-  Log &operator<<(uint32_t n) { return *this << static_cast<uint64_t>(n); }
-  Log &operator<<(uint64_t n);
+  Log &operator<<(short n) { return *this << static_cast<long long>(n); }
+  Log &operator<<(int n) { return *this << static_cast<long long>(n); }
+  Log &operator<<(long n) { return *this << static_cast<long long>(n); }
+  Log &operator<<(long long n);
+  Log &operator<<(unsigned short n) {
+    return *this << static_cast<unsigned long long>(n);
+  }
+  Log &operator<<(unsigned int n) {
+    return *this << static_cast<unsigned long long>(n);
+  }
+  Log &operator<<(unsigned long n) {
+    return *this << static_cast<unsigned long long>(n);
+  }
+  Log &operator<<(unsigned long long n);
   Log &operator<<(float n) { return *this << static_cast<double>(n); }
   Log &operator<<(double n);
   Log &operator<<(long double n);
@@ -172,7 +190,9 @@ public:
     }
   }
   static void set_severity_level(int severity);
-  static int set_severity_level_by_name(const StringRef &name);
+  // Returns the severity level by |name|.  Returns -1 if |name| is
+  // unknown.
+  static int get_severity_level_by_name(const StringRef &name);
   static bool log_enabled(int severity) { return severity >= severity_thres_; }
 
   enum {
@@ -207,43 +227,47 @@ void dec(Log &log);
 #define TTY_HTTP_HD (log_config()->errorlog_tty ? "\033[1;34m" : "")
 #define TTY_RST (log_config()->errorlog_tty ? "\033[0m" : "")
 
-enum LogFragmentType {
-  SHRPX_LOGF_NONE,
-  SHRPX_LOGF_LITERAL,
-  SHRPX_LOGF_REMOTE_ADDR,
-  SHRPX_LOGF_TIME_LOCAL,
-  SHRPX_LOGF_TIME_ISO8601,
-  SHRPX_LOGF_REQUEST,
-  SHRPX_LOGF_STATUS,
-  SHRPX_LOGF_BODY_BYTES_SENT,
-  SHRPX_LOGF_HTTP,
-  SHRPX_LOGF_AUTHORITY,
-  SHRPX_LOGF_REMOTE_PORT,
-  SHRPX_LOGF_SERVER_PORT,
-  SHRPX_LOGF_REQUEST_TIME,
-  SHRPX_LOGF_PID,
-  SHRPX_LOGF_ALPN,
-  SHRPX_LOGF_TLS_CIPHER,
-  SHRPX_LOGF_SSL_CIPHER = SHRPX_LOGF_TLS_CIPHER,
-  SHRPX_LOGF_TLS_PROTOCOL,
-  SHRPX_LOGF_SSL_PROTOCOL = SHRPX_LOGF_TLS_PROTOCOL,
-  SHRPX_LOGF_TLS_SESSION_ID,
-  SHRPX_LOGF_SSL_SESSION_ID = SHRPX_LOGF_TLS_SESSION_ID,
-  SHRPX_LOGF_TLS_SESSION_REUSED,
-  SHRPX_LOGF_SSL_SESSION_REUSED = SHRPX_LOGF_TLS_SESSION_REUSED,
-  SHRPX_LOGF_TLS_SNI,
-  SHRPX_LOGF_TLS_CLIENT_FINGERPRINT_SHA1,
-  SHRPX_LOGF_TLS_CLIENT_FINGERPRINT_SHA256,
-  SHRPX_LOGF_TLS_CLIENT_ISSUER_NAME,
-  SHRPX_LOGF_TLS_CLIENT_SERIAL,
-  SHRPX_LOGF_TLS_CLIENT_SUBJECT_NAME,
-  SHRPX_LOGF_BACKEND_HOST,
-  SHRPX_LOGF_BACKEND_PORT,
+enum class LogFragmentType {
+  NONE,
+  LITERAL,
+  REMOTE_ADDR,
+  TIME_LOCAL,
+  TIME_ISO8601,
+  REQUEST,
+  STATUS,
+  BODY_BYTES_SENT,
+  HTTP,
+  AUTHORITY,
+  REMOTE_PORT,
+  SERVER_PORT,
+  REQUEST_TIME,
+  PID,
+  ALPN,
+  TLS_CIPHER,
+  SSL_CIPHER = TLS_CIPHER,
+  TLS_PROTOCOL,
+  SSL_PROTOCOL = TLS_PROTOCOL,
+  TLS_SESSION_ID,
+  SSL_SESSION_ID = TLS_SESSION_ID,
+  TLS_SESSION_REUSED,
+  SSL_SESSION_REUSED = TLS_SESSION_REUSED,
+  TLS_SNI,
+  TLS_CLIENT_FINGERPRINT_SHA1,
+  TLS_CLIENT_FINGERPRINT_SHA256,
+  TLS_CLIENT_ISSUER_NAME,
+  TLS_CLIENT_SERIAL,
+  TLS_CLIENT_SUBJECT_NAME,
+  BACKEND_HOST,
+  BACKEND_PORT,
+  METHOD,
+  PATH,
+  PATH_WITHOUT_QUERY,
+  PROTOCOL_VERSION,
 };
 
 struct LogFragment {
-  LogFragment(LogFragmentType type, StringRef value = StringRef::from_lit(""))
-      : type(type), value(std::move(value)) {}
+  LogFragment(LogFragmentType type, StringRef value = ""_sr)
+    : type(type), value(std::move(value)) {}
   LogFragmentType type;
   StringRef value;
 };
