@@ -37,7 +37,7 @@ void readcb(struct ev_loop *loop, ev_io *w, int revent) {
 } // namespace
 
 QUICListener::QUICListener(const UpstreamAddr *faddr, Worker *worker)
-    : faddr_{faddr}, worker_{worker} {
+  : faddr_{faddr}, worker_{worker} {
   ev_io_init(&rev_, readcb, faddr_->fd, EV_READ);
   rev_.data = this;
   ev_io_start(worker_->get_loop(), &rev_);
@@ -60,7 +60,7 @@ void QUICListener::on_read() {
   msg.msg_iovlen = 1;
 
   uint8_t msg_ctrl[CMSG_SPACE(sizeof(int)) + CMSG_SPACE(sizeof(in6_pktinfo)) +
-                   CMSG_SPACE(sizeof(uint16_t))];
+                   CMSG_SPACE(sizeof(int))];
   msg.msg_control = msg_ctrl;
 
   auto quic_conn_handler = worker_->get_quic_connection_handler();
@@ -74,6 +74,19 @@ void QUICListener::on_read() {
       return;
     }
 
+    // Packets less than 22 bytes never be a valid QUIC packet.
+    if (nread < 22) {
+      ++pktcnt;
+
+      continue;
+    }
+
+    if (util::quic_prohibited_port(util::get_port(&su))) {
+      ++pktcnt;
+
+      continue;
+    }
+
     Address local_addr{};
     if (util::msghdr_get_local_addr(local_addr, &msg, su.storage.ss_family) !=
         0) {
@@ -85,7 +98,7 @@ void QUICListener::on_read() {
     util::set_port(local_addr, faddr_->port);
 
     ngtcp2_pkt_info pi{
-        .ecn = util::msghdr_get_ecn(&msg, su.storage.ss_family),
+      .ecn = util::msghdr_get_ecn(&msg, su.storage.ss_family),
     };
 
     auto gso_size = util::msghdr_get_udp_gro(&msg);
@@ -93,10 +106,10 @@ void QUICListener::on_read() {
       gso_size = static_cast<size_t>(nread);
     }
 
-    auto data = buf.data();
+    auto data = std::span{std::begin(buf), static_cast<size_t>(nread)};
 
     for (;;) {
-      auto datalen = std::min(static_cast<size_t>(nread), gso_size);
+      auto datalen = std::min(data.size(), gso_size);
 
       ++pktcnt;
 
@@ -108,7 +121,8 @@ void QUICListener::on_read() {
                   << " bytes";
       }
 
-      if (datalen == 0) {
+      // Packets less than 22 bytes never be a valid QUIC packet.
+      if (datalen < 22) {
         break;
       }
 
@@ -117,14 +131,12 @@ void QUICListener::on_read() {
       remote_addr.len = msg.msg_namelen;
 
       quic_conn_handler->handle_packet(faddr_, remote_addr, local_addr, pi,
-                                       data, datalen);
+                                       {std::begin(data), datalen});
 
-      nread -= datalen;
-      if (nread == 0) {
+      data = data.subspan(datalen);
+      if (data.empty()) {
         break;
       }
-
-      data += datalen;
     }
   }
 }
